package org.processmining.plugins.dcnet;

import org.processmining.models.cnet.CNetNode;

interface CNetRelation {

	static class Factory {

		private static final class RelationImpl implements CNetRelation {

			private final CNetNode source;
			private final CNetNode target;

			private RelationImpl(CNetNode source, CNetNode target) {
				this.source = source;
				this.target = target;
			}

			public CNetNode getSource() {
				return source;
			}

			public CNetNode getTarget() {
				return target;
			}

			public int hashCode() {
				final int prime = 31;
				int result = 1;
				result = prime * result + ((source == null) ? 0 : source.hashCode());
				result = prime * result + ((target == null) ? 0 : target.hashCode());
				return result;
			}

			public boolean equals(Object obj) {
				if (this == obj)
					return true;
				if (obj == null)
					return false;
				if (!(obj instanceof Factory.RelationImpl))
					return false;
				Factory.RelationImpl other = (Factory.RelationImpl) obj;
				if (source == null) {
					if (other.source != null)
						return false;
				} else if (!source.equals(other.source))
					return false;
				if (target == null) {
					if (other.target != null)
						return false;
				} else if (!target.equals(other.target))
					return false;
				return true;
			}

			public String toString() {
				return source + "->" + target;
			}

		}

		static CNetRelation create(final CNetNode source, final CNetNode target) {
			return new RelationImpl(source, target);
		}

	}

	CNetNode getSource();

	CNetNode getTarget();

}